@@ -65,6 +65,14 @@ else:
     basestring = str
 
 def extended_iglob(pattern):
+    if '{' in pattern:
+        m = re.match('(.*){([^}]+)}(.*)', pattern)
+        if m:
+            before, switch, after = m.groups()
+            for case in switch.split(','):
+                for path in extended_iglob(before + case + after):
+                    yield path
+            return
     if '**/' in pattern:
         seen = set()
         first, rest = pattern.split('**/', 1)
@@ -85,6 +93,16 @@ def extended_iglob(pattern):
         for path in iglob(pattern):
             yield path
 
+
+def nonempty(it, error_msg="expected non-empty iterator"):
+    empty = True
+    for value in it:
+        empty = False
+        yield value
+    if empty:
+        raise ValueError(error_msg)
+
+
 @cached_function
 def file_hash(filename):
     path = os.path.normpath(filename.encode("UTF-8"))
@@ -603,6 +621,7 @@ def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=Fa
         to_exclude.update(map(os.path.abspath, extended_iglob(pattern)))
 
     module_list = []
+    module_metadata = {}
     for pattern in patterns:
         if isinstance(pattern, str):
             filepattern = pattern
@@ -627,7 +646,7 @@ def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=Fa
         else:
             raise TypeError(pattern)
 
-        for file in extended_iglob(filepattern):
+        for file in nonempty(extended_iglob(filepattern), "'%s' doesn't match any files" % filepattern):
             if os.path.abspath(file) in to_exclude:
                 continue
             pkg = deps.package(file)
@@ -659,7 +678,10 @@ def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=Fa
                         source = encode_filename_in_py2(source)
                         if source not in sources:
                             sources.append(source)
+                    extra_sources = kwds['sources']
                     del kwds['sources']
+                else:
+                    extra_sources = None
                 if 'depends' in kwds:
                     depends = resolve_depends(kwds['depends'], (kwds.get('include_dirs') or []) + [find_root_package_dir(file)])
                     if template is not None:
@@ -674,9 +696,12 @@ def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=Fa
                         name=module_name,
                         sources=sources,
                         **kwds))
+                if extra_sources:
+                    kwds['sources'] = extra_sources
+                module_metadata[module_name] = {'distutils': kwds}
                 m = module_list[-1]
                 seen.add(name)
-    return module_list
+    return module_list, module_metadata
 
 
 # This is the user-exposed entry point.
@@ -719,7 +744,7 @@ def cythonize(module_list, exclude=[], nthreads=0, aliases=None, quiet=False, fo
     cpp_options = CompilationOptions(**options); cpp_options.cplus = True
     ctx = c_options.create_context()
     options = c_options
-    module_list = create_extension_list(
+    module_list, module_metadata = create_extension_list(
         module_list,
         exclude=exclude,
         ctx=ctx,
@@ -791,7 +816,7 @@ def cythonize(module_list, exclude=[], nthreads=0, aliases=None, quiet=False, fo
                     else:
                         fingerprint = None
                     to_compile.append((priority, source, c_file, fingerprint, quiet,
-                                       options, not exclude_failures))
+                                       options, not exclude_failures, module_metadata.get(m.name)))
                 new_sources.append(c_file)
                 if c_file not in modules_by_cfile:
                     modules_by_cfile[c_file] = [m]
@@ -902,7 +927,7 @@ else:
 
 # TODO: Share context? Issue: pyx processing leaks into pxd module
 @record_results
-def cythonize_one(pyx_file, c_file, fingerprint, quiet, options=None, raise_on_failure=True):
+def cythonize_one(pyx_file, c_file, fingerprint, quiet, options=None, raise_on_failure=True, embedded_metadata=None):
     from ..Compiler.Main import compile, default_options
     from ..Compiler.Errors import CompileError, PyrexError
 
@@ -936,6 +961,7 @@ def cythonize_one(pyx_file, c_file, fingerprint, quiet, options=None, raise_on_f
     if options is None:
         options = CompilationOptions(default_options)
     options.output_file = c_file
+    options.embedded_metadata = embedded_metadata
 
     any_failures = 0
     try:
