@@ -102,6 +102,7 @@ EXT_DEP_MODULES = {
     'tag:pstats': 'pstats',
     'tag:posix' : 'posix',
     'tag:array' : 'array',
+    'tag:coverage': 'coverage',
     'tag:ipython': 'IPython',
     'tag:jedi': 'jedi',
 }
@@ -299,45 +300,6 @@ def _is_py3_before_32(excluded, version):
 VER_DEP_MODULES = {
     # tests are excluded if 'CurrentPythonVersion OP VersionTuple', i.e.
     # (2,4) : (operator.lt, ...) excludes ... when PyVer < 2.4.x
-    (2,4) : (operator.lt, lambda x: x in ['run.extern_builtins_T258',
-                                          'run.builtin_sorted',
-                                          'run.reversed_iteration',
-                                          ]),
-    (2,5) : (operator.lt, lambda x: x in ['run.any',
-                                          'run.all',
-                                          'run.yield_from_pep380',  # GeneratorExit
-                                          'run.generator_frame_cycle', # yield in try-finally
-                                          'run.generator_expressions_in_class',
-                                          'run.absolute_import',
-                                          'run.relativeimport_T542',
-                                          'run.relativeimport_star_T542',
-                                          'run.initial_file_path',  # relative import
-                                          'run.pynumber_subtype_conversion',  # bug in Py2.4
-                                          'build.cythonize_script',  # python2.4 -m a.b.c
-                                          'build.cythonize_script_excludes',  # python2.4 -m a.b.c
-                                          'build.cythonize_script_package',  # python2.4 -m a.b.c
-                                          ]),
-    (2,6) : (operator.lt, lambda x: x in ['run.print_function',
-                                          'run.language_level', # print function
-                                          'run.cython3',
-                                          'run.property_decorator_T593', # prop.setter etc.
-                                          'run.generators_py', # generators, with statement
-                                          'run.pure_py', # decorators, with statement
-                                          'run.purecdef',
-                                          'run.struct_conversion',
-                                          'run.bytearray_coercion',
-                                          'run.bytearraymethods',
-                                          'run.bytearray_ascii_auto_encoding',
-                                          'run.bytearray_default_auto_encoding',
-                                          # memory views require buffer protocol
-                                          'memoryview.relaxed_strides',
-                                          'memoryview.cythonarray',
-                                          'memoryview.memslice',
-                                          'memoryview.numpy_memoryview',
-                                          'memoryview.memoryviewattrs',
-                                          'memoryview.memoryview',
-                                          'run.withstat_py',
-                                          ]),
     (2,7) : (operator.lt, lambda x: x in ['run.withstat_py27', # multi context with statement
                                           'run.yield_inside_lambda',
                                           'run.test_dictviews',
@@ -1526,8 +1488,10 @@ class VersionDependencyExcluder:
 
 class FileListExcluder:
 
-    def __init__(self, list_file):
+    def __init__(self, list_file, verbose=False):
+        self.verbose = verbose
         self.excludes = {}
+        self._list_file = list_file
         f = open(list_file)
         try:
             for line in f.readlines():
@@ -1538,7 +1502,12 @@ class FileListExcluder:
             f.close()
 
     def __call__(self, testname, tags=None):
-        return testname in self.excludes or testname.split('.')[-1] in self.excludes
+        exclude = (testname in self.excludes
+                   or testname.split('.')[-1] in self.excludes)
+        if exclude and self.verbose:
+            print("Excluding %s because it's listed in %s"
+                  % (testname, self._list_file))
+        return exclude
 
 class TagsSelector:
 
@@ -1723,7 +1692,7 @@ def main():
                       action="store_true", default=False,
                       help="only compile pyx to c, do not run C compiler or run the tests")
     parser.add_option("--no-refnanny", dest="with_refnanny",
-                      action="store_false", default=True,
+                      action="store_false", default=not IS_PYPY,
                       help="do not regression test reference counting")
     parser.add_option("--no-fork", dest="fork",
                       action="store_false", default=True,
@@ -1782,6 +1751,7 @@ def main():
     parser.add_option("--watermark", dest="watermark", default=None,
                       help="deterministic generated by string")
     parser.add_option("--use_common_utility_dir", default=False, action="store_true")
+    parser.add_option("--use_formal_grammar", default=False, action="store_true")
 
     options, cmd_args = parser.parse_args()
 
@@ -1841,6 +1811,7 @@ def main():
         Options.generate_cleanup_code = 3   # complete cleanup code
         from Cython.Compiler import DebugFlags
         DebugFlags.debug_temp_code_comments = 1
+        pyrex_default_options['formal_grammar'] = options.use_formal_grammar
 
     if options.shard_count > 1 and options.shard_num == -1:
         import multiprocessing
@@ -1941,6 +1912,7 @@ def runtests(options, cmd_args, coverage=None):
                 test_bugs = True
 
     selectors = [ string_selector(r) for r in cmd_args ]
+    verbose_excludes = selectors or options.verbosity >= 2
     if not selectors:
         selectors = [ lambda x, tags=None: True ]
 
@@ -1972,7 +1944,7 @@ def runtests(options, cmd_args, coverage=None):
         exclude_selectors.append(ShardExcludeSelector(options.shard_num, options.shard_count))
 
     if not test_bugs:
-        exclude_selectors += [ FileListExcluder(os.path.join(ROOTDIR, "bugs.txt")) ]
+        exclude_selectors += [ FileListExcluder(os.path.join(ROOTDIR, "bugs.txt"), verbose=verbose_excludes) ]
 
     if sys.platform in ['win32', 'cygwin'] and sys.version_info < (2,6):
         exclude_selectors += [ lambda x: x == "run.specialfloat" ]
